@@ -85,7 +85,6 @@ gdu_utils_configure_file_chooser_for_disk_images (GtkFileChooser *file_chooser,
   g_object_set_data_full (G_OBJECT (file_chooser), "x-gdu-orig-folder", g_strdup (folder), g_free);
   gtk_file_chooser_set_current_folder_uri (file_chooser, folder);
 
-  /* TODO: define proper mime-types */
   if (set_file_types)
     {
       filter = gtk_file_filter_new ();
@@ -97,14 +96,12 @@ gdu_utils_configure_file_chooser_for_disk_images (GtkFileChooser *file_chooser,
         gtk_file_filter_set_name (filter, _("Disk Images (*.img, *.img.xz, *.iso)"));
       else
         gtk_file_filter_set_name (filter, _("Disk Images (*.img, *.iso)"));
-      gtk_file_filter_add_pattern (filter, "*.raw-disk-image");
-      gtk_file_filter_add_pattern (filter, "*.img");
+      gtk_file_filter_add_mime_type (filter, "application/x-raw-disk-image");
       if (allow_compressed)
         {
-          gtk_file_filter_add_pattern (filter, "*.raw-disk-image.xz");
-          gtk_file_filter_add_pattern (filter, "*.img.xz");
+          gtk_file_filter_add_mime_type (filter, "application/x-raw-disk-image-xz-compressed");
         }
-      gtk_file_filter_add_pattern (filter, "*.iso");
+      gtk_file_filter_add_mime_type (filter, "application/x-cd-image");
       gtk_file_chooser_add_filter (file_chooser, filter); /* adopts filter */
       gtk_file_chooser_set_filter (file_chooser, filter);
     }
@@ -780,6 +777,42 @@ gdu_utils_is_ntfs_available (void)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+guint
+gdu_utils_get_max_label_length (const gchar *fstype)
+{
+  guint max_length = G_MAXUINT;
+
+  if (g_strcmp0 (fstype, "exfat") == 0)
+    {
+      max_length = 15;
+    }
+  else if (g_strcmp0 (fstype, "vfat") == 0)
+    {
+      max_length = 11;
+    }
+
+  return max_length;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+gboolean
+_gtk_entry_buffer_truncate_bytes (GtkEntryBuffer *gtk_entry_buffer,
+                                  guint           max_bytes)
+{
+  guint max_utf8_length = max_bytes;
+
+  while (gtk_entry_buffer_get_bytes (gtk_entry_buffer) > max_bytes)
+    {
+      gtk_entry_buffer_delete_text (gtk_entry_buffer, max_utf8_length, -1);
+      max_utf8_length--;
+    }
+
+  return max_utf8_length != max_bytes;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 #ifdef __GNUC_PREREQ
 # if __GNUC_PREREQ(4,6)
 #  pragma GCC diagnostic pop
@@ -859,7 +892,8 @@ static gboolean
 gdu_utils_is_in_use_full (UDisksClient      *client,
                           UDisksObject      *object,
                           UDisksFilesystem **filesystem_to_unmount_out,
-                          UDisksEncrypted  **encrypted_to_lock_out)
+                          UDisksEncrypted  **encrypted_to_lock_out,
+                          gboolean          *last_out)
 {
   UDisksFilesystem *filesystem_to_unmount = NULL;
   UDisksEncrypted *encrypted_to_lock = NULL;
@@ -871,6 +905,7 @@ gdu_utils_is_in_use_full (UDisksClient      *client,
   GList *l;
   GList *objects_to_check = NULL;
   gboolean ret = FALSE;
+  gboolean last = TRUE;
 
   drive = udisks_object_get_drive (object);
   if (drive != NULL)
@@ -943,9 +978,14 @@ gdu_utils_is_in_use_full (UDisksClient      *client,
           const gchar *const *mount_points = udisks_filesystem_get_mount_points (filesystem_for_object);
           if (g_strv_length ((gchar **) mount_points) > 0)
             {
+              if (ret)
+                {
+                  last = FALSE;
+                  break;
+                }
+
               filesystem_to_unmount = g_object_ref (filesystem_for_object);
               ret = TRUE;
-              goto victim_found;
             }
         }
 
@@ -957,14 +997,21 @@ gdu_utils_is_in_use_full (UDisksClient      *client,
           if (cleartext != NULL)
             {
               g_object_unref (cleartext);
+
+              if (ret)
+                {
+                  last = FALSE;
+                  break;
+                }
+
               encrypted_to_lock = g_object_ref (encrypted_for_object);
               ret = TRUE;
-              goto victim_found;
             }
         }
-    }
 
- victim_found:
+      if (ret && last_out == NULL)
+        break;
+    }
 
   if (filesystem_to_unmount_out != NULL)
     {
@@ -976,6 +1023,8 @@ gdu_utils_is_in_use_full (UDisksClient      *client,
       *encrypted_to_lock_out = (encrypted_to_lock != NULL) ?
         g_object_ref (encrypted_to_lock) : NULL;
     }
+  if (last_out != NULL)
+    *last_out = last;
 
   g_clear_object (&partition_table);
   g_list_free_full (partitions, g_object_unref);
@@ -993,7 +1042,7 @@ gboolean
 gdu_utils_is_in_use (UDisksClient *client,
                      UDisksObject *object)
 {
-  return gdu_utils_is_in_use_full (client, object, NULL, NULL);
+  return gdu_utils_is_in_use_full (client, object, NULL, NULL, NULL);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1078,16 +1127,72 @@ unuse_lock_cb (UDisksEncrypted  *encrypted,
     }
 }
 
+static void
+unuse_set_autoclear_cb (UDisksLoop   *loop,
+                        GAsyncResult *res,
+                        gpointer      user_data)
+{
+  UnuseData *data = user_data;
+  GError *error = NULL;
+
+  if (!udisks_loop_call_set_autoclear_finish (loop,
+                                              res,
+                                              &error))
+    {
+      unuse_data_complete (data,
+                           _("Error disabling autoclear for loop device"),
+                           error);
+    }
+  else
+    {
+      unuse_data_iterate (data);
+    }
+}
+
 static void
 unuse_data_iterate (UnuseData *data)
 {
   UDisksObject *object;
   UDisksFilesystem *filesystem_to_unmount = NULL;
   UDisksEncrypted *encrypted_to_lock = NULL;
+  UDisksLoop *loop;
+  UDisksBlock *block;
+  gboolean last;
 
   object = UDISKS_OBJECT (data->object_iter->data);
   gdu_utils_is_in_use_full (data->client, object,
-                            &filesystem_to_unmount, &encrypted_to_lock);
+                            &filesystem_to_unmount, &encrypted_to_lock, NULL);
+  block = udisks_object_peek_block (object);
+
+  if (block != NULL && (filesystem_to_unmount != NULL || encrypted_to_lock != NULL))
+    {
+      loop = udisks_client_get_loop_for_block (data->client, block);
+
+      if (loop != NULL)
+        {
+          if (udisks_loop_get_autoclear (loop))
+            {
+              gdu_utils_is_in_use_full (data->client,
+                                        UDISKS_OBJECT (g_dbus_interface_get_object (G_DBUS_INTERFACE (loop))),
+                                        NULL, NULL, &last);
+              if (last)
+                {
+                  udisks_loop_call_set_autoclear (loop,
+                                                  FALSE,
+                                                  g_variant_new ("a{sv}", NULL),
+                                                  data->cancellable,
+                                                  (GAsyncReadyCallback) unuse_set_autoclear_cb,
+                                                  data);
+                  g_object_unref (loop);
+                  g_clear_object (&encrypted_to_lock);
+                  g_clear_object (&filesystem_to_unmount);
+                  return;
+                }
+            }
+
+          g_object_unref (loop);
+        }
+    }
 
   if (filesystem_to_unmount != NULL)
     {
